/*
 * The Npic library
 *
 * Copyright (C) 2003 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software under the terms of the 
 * GNU Lesser General Public License (LGPL) version 2.1.
*/

/*
 * calc_bresenham.h - 19/10/2008
 *
 * Draw lines with generalized Bresenham's algorithm
*/


#ifndef NPIC__CALC_BRESENHAM_H
#define NPIC__CALC_BRESENHAM_H


/* PUBLIC */

void NpicDrawLine_2c (Npic_image *np, 
        int yP, int xP, 
        int yQ, int xQ, 
        Npic_c cP, Npic_c cQ);
void NpicDrawLine_2l (Npic_image *np, 
        int yP, int xP, 
        int yQ, int xQ, 
        Npic_l cP, Npic_l cQ);
void NpicDrawLine_2d (Npic_image *np, 
        int yP, int xP, 
        int yQ, int xQ, 
        Npic_d cP, Npic_d cQ);
void NpicDrawLine_3c (Npic_image *np, 
        int zP, int yP, int xP, 
        int zQ, int yQ, int xQ, 
        Npic_c cP, Npic_c cQ);
void NpicDrawLine_3l (Npic_image *np, 
        int zP, int yP, int xP, 
        int zQ, int yQ, int xQ, 
        Npic_l cP, Npic_l cQ);
void NpicDrawLine_3d (Npic_image *np, 
        int zP, int yP, int xP, 
        int zQ, int yQ, int xQ, 
        Npic_d cP, Npic_d cQ);
void NpicDrawLine_4c (Npic_image *np, 
        int tP, int zP, int yP, int xP, 
        int tQ, int zQ, int yQ, int xQ, 
        Npic_c cP, Npic_c cQ);
void NpicDrawLine_4l (Npic_image *np, 
        int tP, int zP, int yP, int xP, 
        int tQ, int zQ, int yQ, int xQ, 
        Npic_l cP, Npic_l cQ);
void NpicDrawLine_4d (Npic_image *np, 
        int tP, int zP, int yP, int xP, 
        int tQ, int zQ, int yQ, int xQ, 
        Npic_d cP, Npic_d cQ);
void NpicDrawLine_5c (Npic_image *np, 
        int sP, int tP, int zP, int yP, int xP, 
        int sQ, int tQ, int zQ, int yQ, int xQ, 
        Npic_c cP, Npic_c cQ);
void NpicDrawLine_5l (Npic_image *np, 
        int sP, int tP, int zP, int yP, int xP, 
        int sQ, int tQ, int zQ, int yQ, int xQ, 
        Npic_l cP, Npic_l cQ);
void NpicDrawLine_5d (Npic_image *np, 
        int sP, int tP, int zP, int yP, int xP, 
        int sQ, int tQ, int zQ, int yQ, int xQ, 
        Npic_d cP, Npic_d cQ);
void NpicDrawLine_6c (Npic_image *np, 
        int rP, int sP, int tP, int zP, int yP, int xP, 
        int rQ, int sQ, int tQ, int zQ, int yQ, int xQ, 
        Npic_c cP, Npic_c cQ);
void NpicDrawLine_6l (Npic_image *np, 
        int rP, int sP, int tP, int zP, int yP, int xP, 
        int rQ, int sQ, int tQ, int zQ, int yQ, int xQ, 
        Npic_l cP, Npic_l cQ);
void NpicDrawLine_6d (Npic_image *np, 
        int rP, int sP, int tP, int zP, int yP, int xP, 
        int rQ, int sQ, int tQ, int zQ, int yQ, int xQ, 
        Npic_d cP, Npic_d cQ);
void NpicDrawLine_2q (Npic_image *np, 
        int yP, int xP, 
        int yQ, int xQ, 
        int aP, int bP, int cP, int dP, int aQ, int bQ, int cQ, int dQ);
void NpicDrawLine_3q (Npic_image *np, 
        int zP, int yP, int xP, 
        int zQ, int yQ, int xQ, 
        int aP, int bP, int cP, int dP, int aQ, int bQ, int cQ, int dQ);
void NpicDrawLine_4q (Npic_image *np, 
        int tP, int zP, int yP, int xP, 
        int tQ, int zQ, int yQ, int xQ, 
        int aP, int bP, int cP, int dP, int aQ, int bQ, int cQ, int dQ);
void NpicDrawLine_5q (Npic_image *np, 
        int sP, int tP, int zP, int yP, int xP, 
        int sQ, int tQ, int zQ, int yQ, int xQ, 
        int aP, int bP, int cP, int dP, int aQ, int bQ, int cQ, int dQ);
void NpicDrawLine_6q (Npic_image *np, 
        int rP, int sP, int tP, int zP, int yP, int xP, 
        int rQ, int sQ, int tQ, int zQ, int yQ, int xQ, 
        int aP, int bP, int cP, int dP, int aQ, int bQ, int cQ, int dQ);


/* PRIVATE */


#endif /* NPIC__CALC_BRESENHAM_H */
