/*
 * The Npic library
 *
 * Copyright (C) 2003 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software under the terms of the 
 * GNU Lesser General Public License (LGPL) version 2.1.
*/

/*
 * mask_creat.h - 20/11/2008
 *
 * Masks for distance transforms, medial axis, convolutions, etc.
*/


#ifndef NPIC__MASK_CREAT_H
#define NPIC__MASK_CREAT_H


/* PUBLIC */

Npic_mask *NpicCreateMask (int ntype);
Npic_mask *NpicCreateMaskDP (int dim, int pixel_type);
void NpicDestroyMask (Npic_mask *m);
Npic_mask *NpicCopyMask (Npic_mask *src);

void NpicMaskInsert_2l (Npic_mask *m, 
        int y, int x, Npic_l h);
void NpicMaskInsert_2d (Npic_mask *m, 
        int y, int x, Npic_d h);
void NpicMaskInsert_3l (Npic_mask *m, 
        int z, int y, int x, Npic_l h);
void NpicMaskInsert_3d (Npic_mask *m, 
        int z, int y, int x, Npic_d h);
void NpicMaskInsert_4l (Npic_mask *m, 
        int t, int z, int y, int x, Npic_l h);
void NpicMaskInsert_4d (Npic_mask *m, 
        int t, int z, int y, int x, Npic_d h);
void NpicMaskInsert_5l (Npic_mask *m, 
        int s, int t, int z, int y, int x, Npic_l h);
void NpicMaskInsert_5d (Npic_mask *m, 
        int s, int t, int z, int y, int x, Npic_d h);
void NpicMaskInsert_6l (Npic_mask *m, 
        int r, int s, int t, int z, int y, int x, Npic_l h);
void NpicMaskInsert_6d (Npic_mask *m, 
        int r, int s, int t, int z, int y, int x, Npic_d h);

void *NpicCastMask (Npic_mask *m);
void NpicPrintMask (Npic_mask *m);

/* For backward compatibility; deprecated since 0.17. */
#define NpicInsMask_2l(m,x,y,h)     NpicMaskInsert_2l(m,y,x,h)
#define NpicInsMask_2d(m,x,y,h)     NpicMaskInsert_2d(m,y,x,h)
#define NpicInsMask_3l(m,x,y,z,h)   NpicMaskInsert_3l(m,z,y,x,h)
#define NpicInsMask_3d(m,x,y,z,h)   NpicMaskInsert_3d(m,z,y,x,h)
#define NpicInsMask_4l(m,x,y,z,t,h) NpicMaskInsert_4l(m,t,z,y,x,h)
#define NpicInsMask_4d(m,x,y,z,t,h) NpicMaskInsert_4d(m,t,z,y,x,h)


/* PRIVATE */

int NpicExpandMask (Npic_mask *m, const char *funcname);


#endif /* NPIC__MASK_CREAT_H */
