/* 
    test-formats.c
    
    Edouard.Thiel@lif.univ-mrs.fr - 18/12/2008
    
    Test read/write of image files, depending of extension.
    
    ./test-formats shows usage.

*/

/* DO NOT EDIT !!!  Generated by npic-templa from "test-formats.ct" */


#include <npic.h>

#define ERRFLOAT 1e-3


void test_2c (char *fname, int info)
{
    int y, x;
    Npic_image *np;
    Npic_image_2c *I;
    double time1, time2;

    printf ("---- test_2c ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 200x300 image ... "); fflush (stdout);

    np = NpicCreateImage_2c (
        200, 300, 2, 1);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        I->pix[y][x] = (y*I->xmax+x) % 256;
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        if (I->pix[y][x] != (y*I->xmax+x) % 256) {
            printf ("*** ERROR: bad value in %d %d : %d\n", 
                    y, x, I->pix[y][x]);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_2l (char *fname, int info)
{
    int y, x;
    Npic_image *np;
    Npic_image_2l *I;
    double time1, time2;

    printf ("---- test_2l ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 200x300 image ... "); fflush (stdout);

    np = NpicCreateImage_2l (
        200, 300, 2, 1);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        I->pix[y][x] = (y-100)*I->xmax*123456+x*65795;
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        if (I->pix[y][x] != (y-100)*I->xmax*123456+x*65795) {
            printf ("*** ERROR: bad value in %d %d : %" NPIC_PL "\n", 
                    y, x, I->pix[y][x]);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_2d (char *fname, int info)
{
    int y, x;
    Npic_image *np;
    Npic_image_2d *I;
    double time1, time2;
    Npic_d u;

    printf ("---- test_2d ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 200x300 image ... "); fflush (stdout);

    np = NpicCreateImage_2d (
        200, 300, 2, 1);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        I->pix[y][x] = y*123.7654 + x/65795.98765;
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        u = I->pix[y][x] - (y*123.7654 + x/65795.98765);
        if (u < -ERRFLOAT || u > ERRFLOAT) {
            printf ("*** ERROR: bad value in %d %d : %f %.20f\n", 
                    y, x, I->pix[y][x], u);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_2q (char *fname, int info)
{
    int y, x;
    Npic_image *np;
    Npic_image_2q *I;
    double time1, time2;

    printf ("---- test_2q ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 200x300 image ... "); fflush (stdout);

    np = NpicCreateImage_2q (
        200, 300, 2, 1);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
    {
        I->pix[y][x].a = (y*2001+x*x*656+20000) % 65536;
        I->pix[y][x].b = (y*y*589+x*17000+8760) % 65536;
        I->pix[y][x].c = (y*y*7101+x*x*439+189) % 65536;
        I->pix[y][x].d = (y*I->xmax+x +  11871) % 65536;
    }
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        if ( I->pix[y][x].a != (y*2001+x*x*656+20000) % 65536 ||
             I->pix[y][x].b != (y*y*589+x*17000+8760) % 65536 ||
             I->pix[y][x].c != (y*y*7101+x*x*439+189) % 65536 ||
             I->pix[y][x].d != (y*I->xmax+x  + 11871) % 65536 )
        {
            printf ("*** ERROR: bad value in %d %d : %d %d %d %d\n", 
                    y, x, I->pix[y][x].a, I->pix[y][x].b, I->pix[y][x].c, I->pix[y][x].d);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_3c (char *fname, int info)
{
    int z, y, x;
    Npic_image *np;
    Npic_image_3c *I;
    double time1, time2;

    printf ("---- test_3c ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 80x100x130 image ... "); fflush (stdout);

    np = NpicCreateImage_3c (
        80,100,130, 3, 1, 2);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        I->pix[z][y][x] = (z*I->ymax*I->xmax+y*I->xmax+x) % 256;
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        if (I->pix[z][y][x] != (z*I->ymax*I->xmax+y*I->xmax+x) % 256) {
            printf ("*** ERROR: bad value in %d %d %d : %d\n", 
                    z, y, x, I->pix[z][y][x]);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_3l (char *fname, int info)
{
    int z, y, x;
    Npic_image *np;
    Npic_image_3l *I;
    double time1, time2;

    printf ("---- test_3l ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 80x100x130 image ... "); fflush (stdout);

    np = NpicCreateImage_3l (
        80,100,130, 3, 1, 2);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        I->pix[z][y][x] = z*1234567 - y*21 + x*6543 - 421;
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        if (I->pix[z][y][x] != z*1234567 - y*21 + x*6543 - 421) {
            printf ("*** ERROR: bad value in %d %d %d : %" NPIC_PL "\n", 
                    z, y, x, I->pix[z][y][x]);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_3d (char *fname, int info)
{
    int z, y, x;
    Npic_image *np;
    Npic_image_3d *I;
    double time1, time2;
    Npic_d u;

    printf ("---- test_3d ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 80x100x130 image ... "); fflush (stdout);

    np = NpicCreateImage_3d (
        80,100,130, 3, 1, 2);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        I->pix[z][y][x] = z*2.3156 + y*123.7654 + x/65795.98765;
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        u = I->pix[z][y][x] - (z*2.3156 + y*123.7654 + x/65795.98765);
        if (u < -ERRFLOAT || u > ERRFLOAT) {
            printf ("*** ERROR: bad value in %d %d %d : %f %.20f\n", 
                    z, y, x, I->pix[z][y][x], u);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_3q (char *fname, int info)
{
    int z, y, x;
    Npic_image *np;
    Npic_image_3q *I;
    double time1, time2;

    printf ("---- test_3q ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 80x100x130 image ... "); fflush (stdout);

    np = NpicCreateImage_3q (
        80,100,130, 3, 1, 2);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
    {
        I->pix[z][y][x].a = (y*2001+x*x*656+20000) % 65536;
        I->pix[z][y][x].b = (y*y*589+x*17000+8760) % 65536;
        I->pix[z][y][x].c = (y*y*7101+x*x*439+189) % 65536;
        I->pix[z][y][x].d = (y*I->xmax+x +  11871) % 65536;
    }
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        if ( I->pix[z][y][x].a != (y*2001+x*x*656+20000) % 65536 ||
             I->pix[z][y][x].b != (y*y*589+x*17000+8760) % 65536 ||
             I->pix[z][y][x].c != (y*y*7101+x*x*439+189) % 65536 ||
             I->pix[z][y][x].d != (y*I->xmax+x  + 11871) % 65536 )
        {
            printf ("*** ERROR: bad value in %d %d %d : %d %d %d %d\n", 
                    z, y, x, I->pix[z][y][x].a, I->pix[z][y][x].b, I->pix[z][y][x].c, I->pix[z][y][x].d);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_4c (char *fname, int info)
{
    int t, z, y, x;
    Npic_image *np;
    Npic_image_4c *I;
    double time1, time2;

    printf ("---- test_4c ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 30x50x70x75 image ... "); fflush (stdout);

    np = NpicCreateImage_4c (
        30,50,70,75, 4, 2, 3, 1);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        I->pix[t][z][y][x] = (t*I->zmax*I->ymax*I->xmax+z*I->ymax*I->xmax+y*I->xmax+x) % 255;
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        if (I->pix[t][z][y][x] != (t*I->zmax*I->ymax*I->xmax+
                         z*I->ymax*I->xmax+y*I->xmax+x) % 255) {
            printf ("*** ERROR: bad value in %d %d %d %d : %d\n", 
                    t, z, y, x, I->pix[t][z][y][x]);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_4l (char *fname, int info)
{
    int t, z, y, x;
    Npic_image *np;
    Npic_image_4l *I;
    double time1, time2;

    printf ("---- test_4l ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 30x50x70x75 image ... "); fflush (stdout);

    np = NpicCreateImage_4l (
        30,50,70,75, 4, 2, 3, 1);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        I->pix[t][z][y][x] = t*z*1234567 - y*21 + x*6543 - 421;
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        if (I->pix[t][z][y][x] != t*z*1234567 - y*21 + x*6543 - 421) {
            printf ("*** ERROR: bad value in %d %d %d %d : %" NPIC_PL "\n", 
                    t, z, y, x, I->pix[t][z][y][x]);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_4d (char *fname, int info)
{
    int t, z, y, x;
    Npic_image *np;
    Npic_image_4d *I;
    double time1, time2;
    Npic_d u;

    printf ("---- test_4d ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 30x50x70x75 image ... "); fflush (stdout);

    np = NpicCreateImage_4d (
        30,50,70,75, 4, 2, 3, 1);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        I->pix[t][z][y][x] = t*1.967 - z*2.3156 + y*123.7654 + x/65795.98765;
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        u = I->pix[t][z][y][x] - (t*1.967 - z*2.3156 + y*123.7654 + x/65795.98765);
        if (u < -ERRFLOAT || u > ERRFLOAT) {
            printf ("*** ERROR: bad value in %d %d %d %d : %f %.20f\n", 
                    t, z, y, x, I->pix[t][z][y][x], u);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_4q (char *fname, int info)
{
    int t, z, y, x;
    Npic_image *np;
    Npic_image_4q *I;
    double time1, time2;

    printf ("---- test_4q ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 30x50x70x75 image ... "); fflush (stdout);

    np = NpicCreateImage_4q (
        30,50,70,75, 4, 2, 3, 1);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
    {
        I->pix[t][z][y][x].a = (y*2001+x*x*656+20000) % 65536;
        I->pix[t][z][y][x].b = (y*y*589+x*17000+8760) % 65536;
        I->pix[t][z][y][x].c = (y*y*7101+x*x*439+189) % 65536;
        I->pix[t][z][y][x].d = (y*I->xmax+x +  11871) % 65536;
    }
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        if ( I->pix[t][z][y][x].a != (y*2001+x*x*656+20000) % 65536 ||
             I->pix[t][z][y][x].b != (y*y*589+x*17000+8760) % 65536 ||
             I->pix[t][z][y][x].c != (y*y*7101+x*x*439+189) % 65536 ||
             I->pix[t][z][y][x].d != (y*I->xmax+x  + 11871) % 65536 )
        {
            printf ("*** ERROR: bad value in %d %d %d %d : %d %d %d %d\n", 
                    t, z, y, x, I->pix[t][z][y][x].a, I->pix[t][z][y][x].b, I->pix[t][z][y][x].c, I->pix[t][z][y][x].d);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_5c (char *fname, int info)
{
    int s, t, z, y, x;
    Npic_image *np;
    Npic_image_5c *I;
    double time1, time2;

    printf ("---- test_5c ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 22x25x29x17x22 image ... "); fflush (stdout);

    np = NpicCreateImage_5c (
        22,25,29,17,22, 3, 2, 1, 2, 1);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (s = 0; s < I->smax; s++)
    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        I->pix[s][t][z][y][x] = (s*21+t*I->zmax*I->ymax*I->xmax+z*I->ymax*I->xmax+y*I->xmax+x) % 255;
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (s = 0; s < I->smax; s++)
    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        if (I->pix[s][t][z][y][x] != (s*21+t*I->zmax*I->ymax*I->xmax+
                         z*I->ymax*I->xmax+y*I->xmax+x) % 255) {
            printf ("*** ERROR: bad value in %d %d %d %d %d : %d\n", 
                    s, t, z, y, x, I->pix[s][t][z][y][x]);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_5l (char *fname, int info)
{
    int s, t, z, y, x;
    Npic_image *np;
    Npic_image_5l *I;
    double time1, time2;

    printf ("---- test_5l ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 22x25x29x17x22 image ... "); fflush (stdout);

    np = NpicCreateImage_5l (
        22,25,29,17,22, 3, 2, 1, 2, 1);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (s = 0; s < I->smax; s++)
    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        I->pix[s][t][z][y][x] = s*37+ t*z*1234567 - y*21 + x*6543 - 421;
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (s = 0; s < I->smax; s++)
    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        if (I->pix[s][t][z][y][x] != s*37+t*z*1234567 - y*21 + x*6543 - 421) {
            printf ("*** ERROR: bad value in %d %d %d %d %d : %" NPIC_PL "\n", 
                    s, t, z, y, x, I->pix[s][t][z][y][x]);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_5d (char *fname, int info)
{
    int s, t, z, y, x;
    Npic_image *np;
    Npic_image_5d *I;
    double time1, time2;
    Npic_d u;

    printf ("---- test_5d ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 22x25x29x17x22 image ... "); fflush (stdout);

    np = NpicCreateImage_5d (
        22,25,29,17,22, 3, 2, 1, 2, 1);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (s = 0; s < I->smax; s++)
    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        I->pix[s][t][z][y][x] = s*12.66 + t*1.967 - z*2.3156 + y*123.7654 + x/65795.98765;
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (s = 0; s < I->smax; s++)
    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        u = I->pix[s][t][z][y][x] - (s*12.66 + t*1.967 - z*2.3156 + y*123.7654 + x/65795.98765);
        if (u < -ERRFLOAT || u > ERRFLOAT) {
            printf ("*** ERROR: bad value in %d %d %d %d %d : %f %.20f\n", 
                    s, t, z, y, x, I->pix[s][t][z][y][x], u);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_5q (char *fname, int info)
{
    int s, t, z, y, x;
    Npic_image *np;
    Npic_image_5q *I;
    double time1, time2;

    printf ("---- test_5q ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 22x25x29x17x22 image ... "); fflush (stdout);

    np = NpicCreateImage_5q (
        22,25,29,17,22, 3, 2, 1, 2, 1);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (s = 0; s < I->smax; s++)
    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
    {
        I->pix[s][t][z][y][x].a = (y*2001+x*x*656+20000) % 65536;
        I->pix[s][t][z][y][x].b = (y*y*589+x*17000+8760) % 65536;
        I->pix[s][t][z][y][x].c = (y*y*7101+x*x*439+189) % 65536;
        I->pix[s][t][z][y][x].d = (y*I->xmax+x +  11871) % 65536;
    }
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (s = 0; s < I->smax; s++)
    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        if ( I->pix[s][t][z][y][x].a != (y*2001+x*x*656+20000) % 65536 ||
             I->pix[s][t][z][y][x].b != (y*y*589+x*17000+8760) % 65536 ||
             I->pix[s][t][z][y][x].c != (y*y*7101+x*x*439+189) % 65536 ||
             I->pix[s][t][z][y][x].d != (y*I->xmax+x  + 11871) % 65536 )
        {
            printf ("*** ERROR: bad value in %d %d %d %d %d : %d %d %d %d\n", 
                    s, t, z, y, x, I->pix[s][t][z][y][x].a, I->pix[s][t][z][y][x].b, I->pix[s][t][z][y][x].c, I->pix[s][t][z][y][x].d);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_6c (char *fname, int info)
{
    int r, s, t, z, y, x;
    Npic_image *np;
    Npic_image_6c *I;
    double time1, time2;

    printf ("---- test_6c ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 12x15x18x14x20x11 image ... "); fflush (stdout);

    np = NpicCreateImage_6c (
        12,15,18,14,20,11, 2, 1, 2, 3, 1, 2);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (r = 0; r < I->rmax; r++)
    for (s = 0; s < I->smax; s++)
    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        I->pix[r][s][t][z][y][x] = (r*7+s*17+t*I->zmax*I->ymax*I->xmax+z*I->ymax*I->xmax+y*I->xmax+x) % 255;
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (r = 0; r < I->rmax; r++)
    for (s = 0; s < I->smax; s++)
    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        if (I->pix[r][s][t][z][y][x] != (r*7+s*17+t*I->zmax*I->ymax*I->xmax+
                         z*I->ymax*I->xmax+y*I->xmax+x) % 255) {
            printf ("*** ERROR: bad value in %d %d %d %d %d %d : %d\n", 
                    r, s, t, z, y, x, I->pix[r][s][t][z][y][x]);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_6l (char *fname, int info)
{
    int r, s, t, z, y, x;
    Npic_image *np;
    Npic_image_6l *I;
    double time1, time2;

    printf ("---- test_6l ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 12x15x18x14x20x11 image ... "); fflush (stdout);

    np = NpicCreateImage_6l (
        12,15,18,14,20,11, 2, 1, 2, 3, 1, 2);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (r = 0; r < I->rmax; r++)
    for (s = 0; s < I->smax; s++)
    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        I->pix[r][s][t][z][y][x] = r*s*19+t*z*1234567 - y*21 + x*6543 - 421;
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (r = 0; r < I->rmax; r++)
    for (s = 0; s < I->smax; s++)
    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        if (I->pix[r][s][t][z][y][x] != r*s*19+t*z*1234567 - y*21 + x*6543 - 421) {
            printf ("*** ERROR: bad value in %d %d %d %d %d %d : %" NPIC_PL "\n", 
                    r, s, t, z, y, x, I->pix[r][s][t][z][y][x]);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_6d (char *fname, int info)
{
    int r, s, t, z, y, x;
    Npic_image *np;
    Npic_image_6d *I;
    double time1, time2;
    Npic_d u;

    printf ("---- test_6d ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 12x15x18x14x20x11 image ... "); fflush (stdout);

    np = NpicCreateImage_6d (
        12,15,18,14,20,11, 2, 1, 2, 3, 1, 2);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (r = 0; r < I->rmax; r++)
    for (s = 0; s < I->smax; s++)
    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        I->pix[r][s][t][z][y][x] = r*s*86.4 + t*1.967 - z*2.3156 + y*123.7654 + x/65795.98765;
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (r = 0; r < I->rmax; r++)
    for (s = 0; s < I->smax; s++)
    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        u = I->pix[r][s][t][z][y][x] - (r*s*86.4+t*1.967 - z*2.3156 + y*123.7654 + x/65795.98765);
        if (u < -ERRFLOAT || u > ERRFLOAT) {
            printf ("*** ERROR: bad value in %d %d %d %d %d %d : %f %.20f\n", 
                    r, s, t, z, y, x, I->pix[r][s][t][z][y][x], u);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}


void test_6q (char *fname, int info)
{
    int r, s, t, z, y, x;
    Npic_image *np;
    Npic_image_6q *I;
    double time1, time2;

    printf ("---- test_6q ----\n");
    
    time1 = NpicGetTime();
    printf ("Creating 12x15x18x14x20x11 image ... "); fflush (stdout);

    np = NpicCreateImage_6q (
        12,15,18,14,20,11, 2, 1, 2, 3, 1, 2);

    if (np == NULL) return;
    I = NpicCastImage (np);

    for (r = 0; r < I->rmax; r++)
    for (s = 0; s < I->smax; s++)
    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
    {
        I->pix[r][s][t][z][y][x].a = (y*2001+x*x*656+20000) % 65536;
        I->pix[r][s][t][z][y][x].b = (y*y*589+x*17000+8760) % 65536;
        I->pix[r][s][t][z][y][x].c = (y*y*7101+x*x*439+189) % 65536;
        I->pix[r][s][t][z][y][x].d = (y*I->xmax+x +  11871) % 65536;
    }
        
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Saving image ..... "); fflush (stdout);

    if (NpicWriteImage (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Loading image .... "); fflush (stdout);

    np = NpicReadImage (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1); time1 = time2;
    printf ("Checking image ... "); fflush (stdout);

    for (r = 0; r < I->rmax; r++)
    for (s = 0; s < I->smax; s++)
    for (t = 0; t < I->tmax; t++)
    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++) {
        if ( I->pix[r][s][t][z][y][x].a != (y*2001+x*x*656+20000) % 65536 ||
             I->pix[r][s][t][z][y][x].b != (y*y*589+x*17000+8760) % 65536 ||
             I->pix[r][s][t][z][y][x].c != (y*y*7101+x*x*439+189) % 65536 ||
             I->pix[r][s][t][z][y][x].d != (y*I->xmax+x  + 11871) % 65536 )
        {
            printf ("*** ERROR: bad value in %d %d %d %d %d %d : %d %d %d %d\n", 
                    r, s, t, z, y, x, I->pix[r][s][t][z][y][x].a, I->pix[r][s][t][z][y][x].b, I->pix[r][s][t][z][y][x].c, I->pix[r][s][t][z][y][x].d);
            NpicDestroyImage (np);
            return;
        }
    }
    
    time2 = NpicGetTime (); printf ("%.3f s\n", time2-time1);

    if (info) NpicInfoImage (fname);
    printf ("SUCCESS.\n");

    NpicDestroyImage (np);
}



int main (int argc, char **argv)
{
    int k = 2, i, info = 0;
    
    if (argc < 3) {
        printf ("Usage: %s out [-info] [ -2c -2l -2d -2q"
                    "-3c -3l -3d -3q -4c -4l -4d -4q "
                    "-5c -5l -5d -5q -6c -6l -6d -6q -all ] \n", argv[0]);
        printf ("Extensions allowed: " NPIC_KNOWN_IMAGE_EXT "\n");
        exit(1);
    }
    
    if (strcmp (argv[2], "-info") == 0) { k++; info = 1; }
    for (i = k; i < argc; i++)
    {
        if (strcmp (argv[i], "-2c") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_2c (argv[1], info);
    
        if (strcmp (argv[i], "-2l") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_2l (argv[1], info);
    
        if (strcmp (argv[i], "-2d") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_2d (argv[1], info);
    
        if (strcmp (argv[i], "-2q") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_2q (argv[1], info);
    
        if (strcmp (argv[i], "-3c") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_3c (argv[1], info);
    
        if (strcmp (argv[i], "-3l") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_3l (argv[1], info);
    
        if (strcmp (argv[i], "-3d") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_3d (argv[1], info);
    
        if (strcmp (argv[i], "-3q") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_3q (argv[1], info);
    
        if (strcmp (argv[i], "-4c") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_4c (argv[1], info);
    
        if (strcmp (argv[i], "-4l") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_4l (argv[1], info);
    
        if (strcmp (argv[i], "-4d") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_4d (argv[1], info);
    
        if (strcmp (argv[i], "-4q") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_4q (argv[1], info);
    
        if (strcmp (argv[i], "-5c") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_5c (argv[1], info);
    
        if (strcmp (argv[i], "-5l") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_5l (argv[1], info);
    
        if (strcmp (argv[i], "-5d") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_5d (argv[1], info);
    
        if (strcmp (argv[i], "-5q") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_5q (argv[1], info);
    
        if (strcmp (argv[i], "-6c") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_6c (argv[1], info);
    
        if (strcmp (argv[i], "-6l") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_6l (argv[1], info);
    
        if (strcmp (argv[i], "-6d") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_6d (argv[1], info);
    
        if (strcmp (argv[i], "-6q") == 0 ||
            strcmp (argv[i], "-all") == 0)
            test_6q (argv[1], info);
    
    }
    
    exit(0);
}

