/* 
    test-mirror.c
    
    Edouard.Thiel@lif.univ-mrs.fr - 28/01/2003
    
    Create a NPIC_IMAGE_2L image with external border, write pixel values,
    then call NpicMirrorBorder and show effect.
    Demonstrate also NpicPrintImage and NpicPrintImageB.
    
    USAGE: ./test-mirror
*/

#include <npic.h>


void draw_image_2l (Npic_image *np)
{
    Npic_image_2l *I = NpicCastImage (np);
    int x, y, k = 1;
    
    if (NpicImageIsOK (np, "draw_image_2l") != NPIC_SUCCESS) return;
    
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
    {
        I->pix[y][x] = k++;
    }
}


int main ()
{
    Npic_image *np;
    
    /* Create a _2l image = 2D image with signed long pixels.
       height is ymax=10, width is xmax=5, 
       external borders are ybor=2, xbor=3 
    */
    np = NpicCreateImage_2l (10, 5, 2, 3);
    if (np == NULL) exit (1);
    
    draw_image_2l (np); 
    
    printf ("\nImage:\n");    
    NpicPrintImage (np);
    
    printf ("\nImage and its border:\n");    
    NpicPrintImageB (np);
    
    NpicMirrorBorder (np);
    
    printf ("\nImage and its border after mirror:\n");    
    NpicPrintImageB (np);
    
    NpicDestroyImage (np);
    exit(0);
}

