/* 
    test-npz.c
    
    Edouard.Thiel@lif.univ-mrs.fr - 28/03/2003
    
    Test read/write of NPZ files.
    
    Usage: ./test-npz
*/

#include <npic.h>


int main (int argc, char **argv)
{
    int x, y;
    Npic_image *np;
    Npic_image_2c *I;
    char *fname = "tmp.npz";

    printf ("Creating 200x300 image ...\n");

    np = NpicCreateImage_2c (200, 300, 3, 3);
    if (np == NULL) exit (0);
    I = NpicCastImage (np);

    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        I->pix[y][x] = (y*I->xmax+x) % 256;
    
    printf ("Adding properties ...\n");
    NpicImageAddProp (np, "Colour", "Blue and red : fine !");
    NpicImageAddProp (np, "Background", "White");
    
    printf ("Saving image .....\n");

    if (NpicWriteNPZ (np, fname) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    printf ("Loading image ....\n");

    np = NpicReadNPZ (fname);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);

    printf ("Checking image ...\n");

    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        if (I->pix[y][x] != (y*I->xmax+x) % 256) {
            printf ("*** ERROR: bad value in %d %d : %d\n", 
                    x, y, I->pix[y][x]);
            NpicDestroyImage (np);
            exit (0);
        }
    
    printf ("SUCCESS.\n");
    
    printf ("Printing properties ...\n");
    NpicImagePrintProps (np);

    NpicDestroyImage (np);
 
    exit(0);
}

